/*sr_ints_on_off.c - routines to initilize and turn on or turn off the
	scc (communications controller chip) for receipt of all packets
	(promiscuous listening).

Change log:

  date     who                   what
-------- -------- ---------------------------------------
03/17/87  wohl    don't reference vinstall/remove, seer doesn't use them
				  and by not referencing we can save loading all the
				  system call glue.
02/22/87  wohl    make us a code resource, no a5 globals

 exported routines:
************************************************************************ 
void sr_ints_on(avbl,tranm_int,data_int,special_int,ext_int)
avbl Ptr;			vbl task to install
ProcPtr transm_int,data_int,special_int,ext_int;
 All of the scc initilizing
 takes place with interrupts disabled (via SCC_interupts_off), this is so that
 the scc will be fully initilized and all interrupt routines and the vbl task
 will be installed before the first scc interrupt.
 
 Parameter 'avbl' is a pointer to a vbl task record to install.  It should
 be completely filled out and ready to pass to VInstall.  If no vbl task
 is desired pass NIL for avbl and none will be installed.

 initilize the scc for promiscuous receiving.
   avbl - a pointer to a vbl task block to install in vertical retrace
   		  queue.  Use NIL for avbl if you don't want a vbl task.
   tranm_int - transmit buffer empty interrupt routine.  We suggest
   		using &sr_tranm_int for this parameter.  I never tried transmitting
   		let me know if it works.
   data_int - routine to get an interrupt when a packet begins to arrive.
   		See the nice selection of available packet read routines, such
   		as srnred or srsred defined in companion sr files.
   special_int - routine to handle special interrupts, mostly just
   		used to get the sdlc abort received interrupt that can't be
   		turned off if the data interrupt is on.  We have been happy
   		with using &sr_special_int for this parameter.
   ext_int - external status register change interrupt routine.  We use
   		&sr_ext_int.
   			   
************************************************************************ 
void sr_ints_off()
	Undo a call to sr_ints_on.  sr_ints_on copies the old interrupt vectors
	into some static variables so sr_ints_off can find them to put them
	back.  Also, if sr_ints_on installed a vertical retrace task then
	sr_ints_off will remove it.


************************************************************************ 
void sr_tranm_int()
void sr_special_int()
void sr_ext_int()
	The 'usual' interrupt routines to pass to sr_ints_on

*/

#include <DeviceMgr.h>
#include <VRetraceMgr.h>
#include <OSUtil.h>

#include "seer_interface.h"
#include "seer_private.h"

/*prototypes for internal, not exported functions*/
 /*send a list of control values to the SCC*/
 void scc_wrt_ctl(uint8 *);

/*shutdown port B (undo scc_init values)*/
static uint8 scc_uninit[]={
  9,0x4a,	/*reset B, master interrupt enable, NV*/
 10,0x00,	/*flag under (no abort), flag idle (no abort*/
  0,0x10,	/*reset ext/sts interrupts*/
 15,0x08,	/*enable DCD interrupt (used by the mouse driver)*/
  1,0x01,	/*no receive interrupts, just external interrupts*/
 -1,-1 
};

/*initilize the scc chip port B for AppleTalk usage*/
static uint8 scc_init[]={
  9,0x40,	/*reset channel B, all interrupts off*/
  4,0x20,	/*x1 clock, SDLC mode, syncronious mode, no parity*/
 10,0xe0,	/*CRC preset, FM0 mode, 8 bit sync,flag underun*/
  6,0x00,	/*we are sdlc address zero*/
  7,0x7e,	/*sync character is 7e*/
 12,0x06,	/*timing divisor is divide by 6*/
 13,0x00,	/*high byte of time divisor*/
 14,0xc0,	/*FM mode, baud generator off*/
  3,0xd9,	/*Rx 8 bits/char, hunt mode, Rx CRC on, Rx on*/
  2,0x00,	/*interrupt vector 0*/
 15,0x08,	/*interrupt on sync/hunt*/
  1,0x09,	/*interrupt first char or special, interrupt on external int*/
  9,0x0a,	/*master interrupt enable, VIS (what is a vis????)*/
 11,0x70,	/*receive clk is DPLL, TR clk is BR gen*/
 14,0x21,	/*enter search mode, turn on transmit BR*/
  5,0x60,	/*8 bits/char dtr low,rts low, crc16*/
 0xff,0xff	/*terminate list*/
};

/*send a bunch of control (non data) values to the SCC port B
   data_list is a pointer to pairs of bytes.  the first byte in each
   pair is the internal scc data register number to send to.  the second
   byte is the value to send.
   the list is terminated by a SCC register value of 0xff (the SCC has
   internal registers from 0-15 so 0xff is not a legal register).
*/
static void scc_wrt_ctl(data_list)
register uint8 *data_list;
{register char register_number;
 register char *scc_register;
 scc_register=SCCWr+bCtl;			/*point to SCC write register*/
 while((register_number=*data_list++)>=0) {
  *scc_register = register_number; /*tell the SCC which internal reg we want*/
  *scc_register = *data_list++;	/*send control value to it*/
  }
}


/*scc transmit buffer empty interrupt
   a0 - SCCRd	d0 - free
   a1 - SCCWr	d1 - free
   a2 - free	d2 - free
   a3 - free	d3 - free
*/
void sr_tranm_int()
{
}

/*reset a scc error, called as an interrupt rtn
   a0 - SCCRd	d0 - free
   a1 - SCCWr	d1 - free
   a2 - free	d2 - free
   a3 - free	d3 - free
*/
void sr_special_int()
{asm {
	move.b #0x30,xCtl(a1)	/*reset errors*/
	nop						/*let scc do its thing*/
	move.b #0x38,xCtl(a1)	/*reset ius (interrupt under service)*/
	nop						/*let scc do its thing*/
	move.b #0x20,xCtl(a1)	/*interrupt on the next character*/
	}
}

/*replace system external  interrupt with this
   a0 - SCCRd	d0 - SCC status
   a1 - SCCWr	d1 - SCC status that is different since the last interrupt
   a2 - free	d2 - free
   a3 - free	d3 - free
*/
void sr_ext_int()
{
}

/*sr_ints_off() - set scc B back to uninitilized, undo a
	call to sr_ints_on().
*/
void sr_ints_off()
{SCC_interupts_off;					/*no interrupts while resetting*/

 sd.ast.status=SR_status_off;		/*set our status to off*/

 scc_wrt_ctl(scc_uninit);			/*shut down the SCC*/
 
 /*restore old interrupt vectors*/
 Lvl2DT[Lvl2_B_transm]=(ProcPtr)sd.old_tranm_int;
 Lvl2DT[Lvl2_B_data]=(ProcPtr)sd.old_data_vector;	/*data available int*/
 Lvl2DT[Lvl2_B_special]=(ProcPtr)sd.old_special_vector; /*abort received int*/
 ExtStsDT[Ext_B]=(ProcPtr)sd.old_external_vectors;		/*baud rate zero int*/

#ifdef rubbish
 if(sd.our_vbl!=NIL)				/*have a vertical task?*/
   if(VRemove(sd.our_vbl)!=noErr)	/*yes, remove it*/
   	SysError(11);				/*bomb if VInstall failed*/
#endif
 SCC_interupts_on;
}

/*turn on packet peeking*/
void sr_ints_on(avbl,tranm_int,data_int,special_int,ext_int)
Ptr avbl;			/*vbl task to install*/
rtn_ptr tranm_int,data_int,special_int,ext_int;
{sd.our_vbl=(char *)avbl; /*remember our vbl task so we can remove it later*/
 /*don't allow a half initilized scc to interupt us*/
 SCC_interupts_off;

 sd.ast.status=SR_status_on;		/*set our status to on*/

#ifdef rubbish
 if(sd.our_vbl!=NIL)				/*have a vertical task?*/
   if(VInstall(sd.our_vbl)!=noErr)	/*yes, install it*/
   	SysError(11);					/*bomb if VInstall failed*/
#endif
 
 /*save old interrupt vectors*/
 sd.old_tranm_int=(rtn_ptr)Lvl2DT[Lvl2_B_transm];
 sd.old_data_vector=(rtn_ptr)Lvl2DT[Lvl2_B_data];
 sd.old_special_vector=(rtn_ptr)Lvl2DT[Lvl2_B_special];
 sd.old_external_vectors=(rtn_ptr)ExtStsDT[Ext_B];

 /*change interrupt vectors to come to our routines*/
 Lvl2DT[Lvl2_B_transm]=(ProcPtr)tranm_int;
 Lvl2DT[Lvl2_B_data]=(ProcPtr)data_int;
 Lvl2DT[Lvl2_B_special]=(ProcPtr)special_int;
 ExtStsDT[Ext_B]=(ProcPtr)ext_int;
 scc_wrt_ctl(scc_init);			/*fire up the SCC*/
 
 SCC_interupts_on;
}
