/*sr_readpacket.c - seer normal read packet routine
*/

#include "seer_interface.h"
#include "seer_private.h"

/*
	This table maps the three hardware error bits into a single error
	code.  The three hardware bits are 1)overrun, 2)abort, 3)crc.
	If there is an overrun then the other two bits are meaningless.
	If there is no abort crc is meaningless (garbage).
	If there is an abort flag, then crc should be checked.

	There are three status bits, use them as in interger index into
	this table to get a proper error code.  Note: the scc keeps the
	bits at the left end of the status byte so shift them over to get
	an index between 0 thru 7.

*/
static uint16 pak_err_cod[8]={
	PE_noabort,		/*0 timeout no abort */
	PE_overrun,		/*1 data overrun */
	PE_noabort,		/*2 no abort+crc say no abort */
	PE_overrun,		/*3 bad crc+overrun, overrun is only valid flag */
	PE_none,		/*4 just an abort, fine */
	PE_overrun,		/*5 abort and overun?, say overrun */
	PE_badcrc,		/*6 abort and crc means bad crc */
	PE_overrun		/*7 overun takes precidence */
};


/*SCC is receiving a data packet for us
   a0 - SCCRd	d0 - free
   a1 - SCCWr	d1 - free
   a2 - free	d2 - free
   a3 - free	d3 - free
*/
void sr_readpacket()
{iSETUP_GLOBAL_REG;		/*point to data */

#define sccR a0			/*interrupting scc read address */
#define sccW a1			/*interrupting scc write address */
#define packet a2		/*a into pointer into the packet being received */
#define errflag d2		/*error flags */
#define used d3			/*bytes used in packet */
 asm {
	move.l sd.dq.bufwritept,packet /*free block header of where to write */
	moveq #0,used			/*no bytes have been read yet */
	moveq #0,errflag		/*no errors in this packet yet */

have_data:					/*a character is ready to read */
	move.b #1,(sccW)		/*address status reg */
	move.w #0xa0,d1 		/*keep end frame and rx overun, and crc */
	and.b (sccR),d1			/*get status */
	nop
 	move.b bData(sccR),d0	/*read it */
	cmp.l #PAK_SIZE,used	/*room for one more */
	bge.s @data_exit		/*packet is full */
	add.l #1,used			/*count packets */
	move.b d0,(packet)+		/*record received packet */
  	tst.w d1
  	beq.s @read_data		/*get more data if there isn't an error */
 	or.w d1,errflag			/*remember errors in this packet */
 	move.b 0x30,(sccW)		/*reset error condition */
	bra.s @data_exit

read_data:
	moveq #70,d0			/*wait this long between characters */
read_wait_for_char:
	btst #0,(sccR)			/*is a character ready to read? */
 	bne.s @have_data		/*yes, go read it */
 	dbra d0,@read_wait_for_char /*no try again */
/*	bra.s @data_exit */

data_exit:
 	move.b 0x30,(sccW)		/*reset error condition */
	nop
	move.b #0x38,(sccW)		/*reset ius */
	nop
	move.b #0x20,(sccW)		/*interrupt on the next character */

/*a0/a1 used as scratch below here */

/*
	reformat the error bits to be an error number.
*/
	asr #4,errflag			/*make the error bits an index into pak_err_cod*/
	/*errflag = 2* index into pak_err_cod */
	lea pak_err_cod,a0
	move.w 0(a0,errflag.l),errflag
	
#undef sccR		/*free register a0*/
#undef sccW		/*free register a1*/

	move.l sd.dq.bufwritept,a3 	/*start of the packet */
	move.l sd.ast.filter_proc,a0 /*get the users per packet filter routine*/
	lea.l sd,a1				/*filter may want our globals*/
	jsr (a0)				/*want to keep this packet?*/
	bne @read_exit			/*no, leave the queue alone then */

	move.w sd.ast.status,d0	/*is recording on? */
	cmp.w #SR_status_on,d0	/*status set to on? */
	bne @read_exit			/*no, leave the queue alone then */

/*handle an odd length packet, round up packet pointer so the next
   packet starts on an even boundry.  Also set srphfl_ODD in the
   status byte so the real length can be computed.
*/
	btst #0,used			/*an odd length packet? */
	beq.s @notodd			/*no adjustment needed if no */
	add.l #1,packet			/*round up to make it even */
	add.l #1,used
	or #srphfl_ODD,errflag /*record the status */
notodd:

/*
			below here a0 is the new sd.dq.bufwritept
*/

/*handle queueing of packet header */
	move.l sd.hq.bufwritept,a0 	/*where to put this queue entry */
	move.l sd.dq.bufwritept,d0 	/*start of the packet */
	move.l d0,(a0)+				/*store the 24 bit address of received data */
	move.b errflag,-4(a0)		/*put status in high byte */
	cmpa.l sd.hq.bufendpt,a0 	/*hit the end of the queue ? */
	bne.s @sdwrap				/*jump if no need to wrap the pointers */
	move.l sd.hq.bufstartpt,a0 	/*the next packet goes here */
sdwrap:
	cmpa.l sd.hq.bufreadpt,a0 	/*is the queue full? */
	beq.s @hqfull				/*jump if queue full */

/*a0 - new sd.hq.bufwritept */
/*handle queueing of packet data */
	move.l sd.dq.buflength,d0	/*figure the amount of space left */
	sub.l sd.ast.d_inuse,d0
	sub.l #(2*PAK_SIZE),d0		/*see if there is room for two more */
	bmi.s @dqfull

	move.l sd.dq.bufendpt,d0	/*get space till wrap */
	sub.l packet,d0				/*see if we need to wrap the write pointer */
	cmp.l #PAK_SIZE,d0
	bpl.s @qok					/*jump if no need to wrap */
	add.l d0,used				/*count extra on the end as in use */
	or #srphfl_WRAP,errflag		/*remember to get length from final */
	move.l packet,sd.dq_final 	/*remember for length of last packet */
	move.l sd.dq.bufstartpt,packet /*start putting packets here */
	bra.s @qok

hqfull:
	move.b #SR_status_hfull,sd.ast.status /*tell the user that the queue is full */
	bra.s @read_exit
dqfull:
	move.b #SR_status_dfull,sd.ast.status /*tell the user that the queue is full */
	bra.s @read_exit

/****************** exit routines *****************/

/*accept a packet
	a0 - new hq.bufwritept
	packet - new dq.bufwritept
	used - space used
*/
qok:
	move.l a0,sd.hq.bufwritept 		/*update the header write pointer */
	move.l packet,sd.dq.bufwritept /*update the data write pointer */
	add.l #1,sd.ast.h_inuse
	add.l used,sd.ast.d_inuse
/*	bra.s @read_exit */
read_exit:
	}
 iRESTORE_GLOBAL_REG;
}

